@@ -28,6 +28,7 @@
 #endif
 
 #include <stdio.h>
+#include <stdbool.h>
 #include <errno.h>
 #include <ctype.h>
 #include <fcntl.h>
@@ -48,6 +49,9 @@
 
 #define NIBBLE_TO_ASCII(c)  ((c) < 0x0a ? (c) + 0x30 : (c) + 0x57)
 
+#define BREDR_DEFAULT_PSM	0x1011
+#define LE_DEFAULT_PSM		0x0080
+
 /* Test modes */
 enum {
 	SEND,
@@ -87,7 +91,7 @@ static long buffer_size = 2048;
 
 /* Default addr and psm and cid */
 static bdaddr_t bdaddr;
-static unsigned short psm = 0x1011;
+static unsigned short psm = 0;
 static unsigned short cid = 0;
 
 /* Default number of frames to send (-1 = infinite) */
@@ -258,6 +262,37 @@ static void hexdump(unsigned char *s, unsigned long l)
 	}
 }
 
+static int getopts(int sk, struct l2cap_options *opts, bool connected)
+{
+	socklen_t optlen;
+	int err;
+
+	memset(opts, 0, sizeof(*opts));
+
+	if (bdaddr_type == BDADDR_BREDR || cid) {
+		optlen = sizeof(*opts);
+		return getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, opts, &optlen);
+	}
+
+	optlen = sizeof(opts->imtu);
+	err = getsockopt(sk, SOL_BLUETOOTH, BT_RCVMTU, &opts->imtu, &optlen);
+	if (err < 0 || !connected)
+		return err;
+
+	optlen = sizeof(opts->omtu);
+	return getsockopt(sk, SOL_BLUETOOTH, BT_SNDMTU, &opts->omtu, &optlen);
+}
+
+static int setopts(int sk, struct l2cap_options *opts)
+{
+	if (bdaddr_type == BDADDR_BREDR || cid)
+		return setsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, opts,
+								sizeof(opts));
+
+	return setsockopt(sk, SOL_BLUETOOTH, BT_RCVMTU, &opts->imtu,
+							sizeof(opts->imtu));
+}
+
 static int do_connect(char *svr)
 {
 	struct sockaddr_l2 addr;
@@ -280,6 +315,8 @@ static int do_connect(char *svr)
 	addr.l2_family = AF_BLUETOOTH;
 	bacpy(&addr.l2_bdaddr, &bdaddr);
 	addr.l2_bdaddr_type = bdaddr_type;
+	if (cid)
+		addr.l2_cid = htobs(cid);
 
 	if (bind(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
 		syslog(LOG_ERR, "Can't bind socket: %s (%d)",
@@ -288,12 +325,9 @@ static int do_connect(char *svr)
 	}
 
 	/* Get default options */
-	memset(&opts, 0, sizeof(opts));
-	optlen = sizeof(opts);
-
-	if (getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &opts, &optlen) < 0) {
+	if (getopts(sk, &opts, false) < 0) {
 		syslog(LOG_ERR, "Can't get default L2CAP options: %s (%d)",
-							strerror(errno), errno);
+						strerror(errno), errno);
 		goto error;
 	}
 
@@ -306,7 +340,7 @@ static int do_connect(char *svr)
 	opts.txwin_size = txwin_size;
 	opts.max_tx = max_transmit;
 
-	if (setsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &opts, sizeof(opts)) < 0) {
+	if (setopts(sk, &opts) < 0) {
 		syslog(LOG_ERR, "Can't set L2CAP options: %s (%d)",
 							strerror(errno), errno);
 		goto error;
@@ -398,10 +432,7 @@ static int do_connect(char *svr)
 	}
 
 	/* Get current options */
-	memset(&opts, 0, sizeof(opts));
-	optlen = sizeof(opts);
-
-	if (getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &opts, &optlen) < 0) {
+	if (getopts(sk, &opts, true) < 0) {
 		syslog(LOG_ERR, "Can't get L2CAP options: %s (%d)",
 							strerror(errno), errno);
 		goto error;
@@ -529,10 +560,7 @@ static void do_listen(void (*handler)(int sk))
 	}
 
 	/* Get default options */
-	memset(&opts, 0, sizeof(opts));
-	optlen = sizeof(opts);
-
-	if (getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &opts, &optlen) < 0) {
+	if (getopts(sk, &opts, false) < 0) {
 		syslog(LOG_ERR, "Can't get default L2CAP options: %s (%d)",
 							strerror(errno), errno);
 		goto error;
@@ -548,7 +576,7 @@ static void do_listen(void (*handler)(int sk))
 	opts.txwin_size = txwin_size;
 	opts.max_tx = max_transmit;
 
-	if (setsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &opts, sizeof(opts)) < 0) {
+	if (setopts(sk, &opts) < 0) {
 		syslog(LOG_ERR, "Can't set L2CAP options: %s (%d)",
 							strerror(errno), errno);
 		goto error;
@@ -627,10 +655,7 @@ static void do_listen(void (*handler)(int sk))
 		}
 
 		/* Get current options */
-		memset(&opts, 0, sizeof(opts));
-		optlen = sizeof(opts);
-
-		if (getsockopt(nsk, SOL_L2CAP, L2CAP_OPTIONS, &opts, &optlen) < 0) {
+		if (getopts(nsk, &opts, true) < 0) {
 			syslog(LOG_ERR, "Can't get L2CAP options: %s (%d)",
 							strerror(errno), errno);
 			if (!defer_setup) {
@@ -1523,6 +1548,13 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if (!psm) {
+		if (bdaddr_type == BDADDR_BREDR)
+			psm = BREDR_DEFAULT_PSM;
+		else
+			psm = LE_DEFAULT_PSM;
+	}
+
 	if (need_addr && !(argc - optind)) {
 		usage();
 		exit(1);
